Config

Trait Config 

pub trait Config: ConfigCore {
    type LengthEncoding: SeqLen<Self> + 'static;
    type TagEncoding: TagEncoding<Self> + 'static;
}
Trait for configuration access when you need access to type parameters that depend on Config (e.g., Config::LengthEncoding).

Prefer ConfigCore when you don’t need those configuration type parameters that depend on Config (e.g., primitive types).

Required Associated Types

type LengthEncoding: SeqLen<Self> + 'static

type TagEncoding: TagEncoding<Self> + 'static

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors

Source

impl<const ZERO_COPY_ALIGN_CHECK: bool, const PREALLOCATION_SIZE_LIMIT: usize, LengthEncoding, B, I, T> Config for Configuration<ZERO_COPY_ALIGN_CHECK, PREALLOCATION_SIZE_LIMIT, LengthEncoding, B, I, T>
where LengthEncoding: SeqLen<Self> + 'static, T: TagEncoding<Self>, B: ByteOrder, I: IntEncoding<B>,

Source

type LengthEncoding = LengthEncoding

Source

type TagEncoding = T