pub fn serialize_into<T>(dst: impl Writer, src: &T) -> WriteResult<()>Expand description
Serialize a type into the given writer.
Like serialize, but allows the caller to provide their own writer.
pub fn serialize_into<T>(dst: impl Writer, src: &T) -> WriteResult<()>Serialize a type into the given writer.
Like serialize, but allows the caller to provide their own writer.