TrustedSliceWriter

Struct TrustedSliceWriter 

pub struct TrustedSliceWriter<'a> { /* private fields */ }
In-memory Writer that does not perform bounds checking.

Generally this should not be constructed directly, but rather by calling Writer::as_trusted_for on a trusted Writer. This will ensure that the safety invariants are upheld.

Safety

  • The inner buffer must have sufficient capacity for all writes. It is UB if this is not upheld.