BincodeLen

Struct BincodeLen 

pub struct BincodeLen<const MAX_SIZE: usize = DEFAULT_BINCODE_LEN_MAX_SIZE>;
SeqLen implementation for bincode’s default fixint encoding.

The MAX_SIZE constant is a limit on the maximum preallocation size (in bytes) for heap allocated structures. This is a safety precaution against malicious input causing OOM. The default is 4 MiB. Users are free to override this limit by passing a different constant or by implementing their own SeqLen implementation.